import pystan
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sb
import scipy.stats as ss
import csv
import pickle

fin_nm = input('Data(*.csv) = ')
with open(fin_nm, 'r') as f:
    raw_data = [v for v in csv.reader(f)]
#print(raw_data)

N_st = int(raw_data[0][0])

N_data = 0
ID_i = []
ID_j = []
ID_s = []
ID_t = []
N = []
TN = []
for v in raw_data[2:]:
    N_data += 1
    ID_i.append(int(v[0]))
    ID_j.append(int(v[1]))
    ID_s.append(int(v[2]))
    ID_t.append(int(v[3]))
    N.append(int(v[4]))
    TN.append(int(v[5]))

print('N_data = ', N_data)

for i in range(N_data):
    print("{0:>3}: {1}, {2}, {3}, {4}/  {5}, {6}".
          format((i + 1), ID_i[i], ID_j[i], ID_s[i], ID_t[i], N[i], TN[i]))

#sys.exit()
    
Data = {'N_data': N_data, 'N_st': N_st, \
        'ID_i': ID_i, 'ID_j': ID_j, 'ID_s': ID_s, 'ID_t': ID_t, \
        'N': N, 'TN': TN}


sm = pystan.StanModel(file = 'ba_diff_s.stan')

with open('sm.pkl', 'wb') as f:
    pickle.dump(sm, f)


with open('sm.pkl', 'rb') as f:
    sm = pickle.load(f)


fit = sm.sampling(data = Data, n_jobs = 1)

with open('fit.pkl', 'wb') as f:
    pickle.dump(fit, f)


with open('fit.pkl', 'rb') as f:
    fit = pickle.load(f)

print(fit)

Psi = fit['psi'].T
plt.plot([0, 0], [0, 10], label = '$\psi1$')
for i in range(1, N_st-1):
    sb.kdeplot(Psi[i], label = r'$\psi${}'.format(i+1))
plt.plot([1, 1], [0, 10], label = r'$\psi${}'.format(N_st))
plt.legend()
plt.yticks([])
plt.xlabel('$\psi$', fontsize = 16)
plt.title('Posterior Distributions', fontsize = 18)
plt.savefig('FigPsi.png')
plt.show()

Sgm = fit['sgm']
sb.kdeplot(Sgm)
plt.xlabel('$\psi$', fontsize = 14)
plt.title('Posterir distribution of $\sigma$', fontsize = 18)
plt.yticks([])
plt.savefig('FigSgm.png')
plt.show()

Sgm_med = np.percentile(Sgm, 50)
print('Sgm_med =', Sgm_med)


Psi_med = []
Psi_025 = []
Psi_25 = []
Psi_75 = []
Psi_975 = []

for i in range(N_st):
    q025, q25, med, q75, q975 = np.percentile(Psi[i], [2.5, 25, 50, 75, 97.5])
    Psi_med.append(med)
    Psi_025.append(q025)
    Psi_25.append(q25)
    Psi_75.append(q75)
    Psi_975.append(q975)
    
    
stimuli = []
for i in range(N_st):
    stimuli.append(i + 1)


plt.plot(stimuli, Psi_med, 'b-', linewidth = 2, label = 'Median')
plt.plot(stimuli, Psi_25, 'g-.', linewidth = 1, label = '50% CI')
plt.plot(stimuli, Psi_75, 'g-.', linewidth = 1)
plt.plot(stimuli, Psi_025, 'y--', linewidth = 1, label = '95% CI')
plt.plot(stimuli, Psi_975, 'y--', linewidth = 1)
plt.legend()  
plt.title('Sensation and Stimulus', fontsize = 18)
plt.xlabel('Stimulus', fontsize = 14)
plt.ylabel('Sensation', fontsize = 14)
plt.tight_layout()
plt.savefig('FigSummary.png')
plt.show()

Prop_data = np.empty(N_data)
Prop_model = np.empty(N_data)
for i in range(N_data):
    Prop_data[i] = N[i] / TN[i]
    Prop_model[i] = ss.norm(0, 1).cdf((abs(Psi_med[ID_i[i]-1] - Psi_med[ID_j[i]-1]) -
                                   abs(Psi_med[ID_s[i]-1] - Psi_med[ID_t[i]-1])) / Sgm_med)

plt.figure(figsize = (5,5))
plt.scatter(Prop_model, Prop_data)
plt.plot([0,1], [0,1])
plt.title('Scattergram of (Prob_Model, Prop_Data)', fontsize = 16)
plt.ylabel('N/TN', fontsize = 14)
plt.xlabel('Prob.', fontsize = 16)
plt.tight_layout()
plt.savefig('FigScatter.png')
plt.show()
